@@ -5,7 +5,7 @@
  *
  * Written by:
  *	Jan Nijtmans
- *	CMG (Computer Management Group) Arnhem B.V.
+ *	CMG Oost-Nederland B.V.
  *	email: j.nijtmans@chello.nl (private)
  *	       jan.nijtmans@cmg.nl (work)
  *	url:   http://purl.oclc.org/net/nijtmans/
@@ -41,11 +41,11 @@ typedef struct ParseInfo {
  * The format record for the XBM file format:
  */
 
-static int		ChnMatchXBM _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Channel chan,
+static int		ChnMatchXBM _ANSI_ARGS_((Tcl_Channel chan,
 			    Tcl_Obj *fileName, Tcl_Obj *format, int *widthPtr,
-			    int *heightPtr));
-static int      	ObjMatchXBM _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *data,
-		            Tcl_Obj *format, int *widthPtr, int *heightPtr));
+			    int *heightPtr, Tcl_Interp *interp));
+static int      	ObjMatchXBM _ANSI_ARGS_((Tcl_Obj *data,
+		            Tcl_Obj *format, int *widthPtr, int *heightPtr, Tcl_Interp *interp));
 static int		ChnReadXBM  _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Channel chan, Tcl_Obj *fileName,
 			    Tcl_Obj *format, Tk_PhotoHandle imageHandle,
@@ -59,8 +59,7 @@ static int              ChnWriteXBM _ANSI_ARGS_((Tcl_Interp *interp,
                             char *fileName, Tcl_Obj *format,
                             Tk_PhotoImageBlock *blockPtr));
 static int	        StringWriteXBM _ANSI_ARGS_((Tcl_Interp *interp,
-               		    Tcl_DString *dataPtr, Tcl_Obj *format,
-		            Tk_PhotoImageBlock *blockPtr));
+		            Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
 
 static int		CommonReadXBM _ANSI_ARGS_((Tcl_Interp *interp,
 			    ParseInfo *parseInfo,
@@ -107,13 +106,13 @@ static int	NextBitmapWord _ANSI_ARGS_((ParseInfo *parseInfoPtr));
  *----------------------------------------------------------------------
  */
 static int
-ObjMatchXBM(interp, data, format, widthPtr, heightPtr)
-    Tcl_Interp *interp;
+ObjMatchXBM(data, format, widthPtr, heightPtr, interp)
     Tcl_Obj *data;		/* The data supplied by the image */
     Tcl_Obj *format;		/* User-specified format string, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * raw XBM file. */
+    Tcl_Interp *interp;
 {
     ParseInfo parseInfo;
 
@@ -145,14 +144,14 @@ ObjMatchXBM(interp, data, format, widthPtr, heightPtr)
  */
 
 static int
-ChnMatchXBM(interp, chan, fileName, format, widthPtr, heightPtr)
-    Tcl_Interp *interp;
+ChnMatchXBM(chan, fileName, format, widthPtr, heightPtr, interp)
     Tcl_Channel chan;		/* The image channel, open for reading. */
     Tcl_Obj *fileName;		/* The name of the image file. */
     Tcl_Obj *format;		/* User-specified format object, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * raw XBM file. */
+    Tcl_Interp *interp;
 {
     ParseInfo parseInfo;
 
@@ -163,7 +162,7 @@ ChnMatchXBM(interp, chan, fileName, format, widthPtr, heightPtr)
 
     return ReadXBMFileHeader(&parseInfo, widthPtr, heightPtr);
 }
-
+
 
 /*
  *----------------------------------------------------------------------
@@ -266,7 +265,7 @@ CommonReadXBM(interp, parseInfo, format, imageHandle, destX, destY,
     ckfree((char *) data);
     return TCL_OK;
 }
-
+
 
 /*
  *----------------------------------------------------------------------
@@ -420,7 +419,7 @@ NextBitmapWord(parseInfoPtr)
 	*dst = buf;
 	dst++;
 	parseInfoPtr->wordLength++;
-	if (parseInfoPtr->wordLength > MAX_WORD_LENGTH) {
+	if (num == 0 || parseInfoPtr->wordLength > MAX_WORD_LENGTH) {
 	    return TCL_ERROR;
 	}
     }
@@ -566,15 +565,15 @@ ChnWriteXBM(interp, fileName, format, blockPtr)
  *
  *----------------------------------------------------------------------
  */
-static int	
-StringWriteXBM(interp, dataPtr, format, blockPtr)
+static int
+StringWriteXBM(interp, format, blockPtr)
     Tcl_Interp *interp;
-    Tcl_DString *dataPtr;
     Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     int result;
     Tcl_DString data;
+    Tcl_DString *dataPtr;
 
     ImgFixStringWriteProc(&data, &interp, &dataPtr, &format, &blockPtr);
 
@@ -619,7 +618,7 @@ CommonWriteXBM(interp, fileName, dataPtr, format, blockPtr)
     Tcl_Channel chan = (Tcl_Channel) NULL;
     char buffer[256];
     unsigned char *pp;
-    int x, y, i, value, mask;
+    int x, y, value, mask;
     int sep = ' ';
     int alphaOffset;
     char *p = NULL;
@@ -687,8 +686,7 @@ static char %s_bits[] = {\n";
 	    } else {
 		/* make transparent pixel */
 	    }
-	    pp += blockPtr->pixelSize;	
-	    i++;
+	    pp += blockPtr->pixelSize;
 	    mask <<= 1;
 	    if (mask >= 256)
              {
