@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkButton.c,v 1.20 2002/08/08 09:35:34 hobbs Exp $
+ * RCS: @(#) $Id: tkButton.c,v 1.20.2.2 2003/11/12 00:05:13 hobbs Exp $
  */
 #include "tkButton.h"
 #include "default.h"
@@ -135,8 +135,8 @@ static Tk_OptionSpec labelOptionSpecs[] = {
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING, "-text", "text", "Text",
 	DEF_BUTTON_TEXT, Tk_Offset(TkButton, textPtr), -1, 0, 0, 0},
-    {TK_OPTION_STRING, "-textvariable", "textVariable", "Variable",
-	DEF_BUTTON_TEXT_VARIABLE, Tk_Offset(TkButton, textVarNamePtr), -1,
+    {TK_OPTION_SCALARVAR, "-textvariable", "textVariable", "Variable",
+	DEF_BUTTON_TEXT_VARIABLE, -1, Tk_Offset(TkButton, textVarNamePtr),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_INT, "-underline", "underline", "Underline",
 	DEF_BUTTON_UNDERLINE, -1, Tk_Offset(TkButton, underline), 0, 0, 0},
@@ -171,8 +171,8 @@ static Tk_OptionSpec buttonOptionSpecs[] = {
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
 	DEF_BUTTON_BORDER_WIDTH, Tk_Offset(TkButton, borderWidthPtr),
 	Tk_Offset(TkButton, borderWidth), 0, 0, 0},
-    {TK_OPTION_STRING, "-command", "command", "Command",
-	DEF_BUTTON_COMMAND, Tk_Offset(TkButton, commandPtr), -1,
+    {TK_OPTION_CALLBACK, "-command", "command", "Command",
+	DEF_BUTTON_COMMAND, -1, Tk_Offset(TkButton, commandPtr),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING_TABLE, "-compound", "compound", "Compound",
 	 DEF_BUTTON_COMPOUND, -1, Tk_Offset(TkButton, compound), 0,
@@ -237,8 +237,8 @@ static Tk_OptionSpec buttonOptionSpecs[] = {
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING, "-text", "text", "Text",
 	DEF_BUTTON_TEXT, Tk_Offset(TkButton, textPtr), -1, 0, 0, 0},
-    {TK_OPTION_STRING, "-textvariable", "textVariable", "Variable",
-	DEF_BUTTON_TEXT_VARIABLE, Tk_Offset(TkButton, textVarNamePtr), -1,
+    {TK_OPTION_SCALARVAR, "-textvariable", "textVariable", "Variable",
+	DEF_BUTTON_TEXT_VARIABLE, -1, Tk_Offset(TkButton, textVarNamePtr),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_INT, "-underline", "underline", "Underline",
 	DEF_BUTTON_UNDERLINE, -1, Tk_Offset(TkButton, underline), 0, 0, 0},
@@ -273,8 +273,8 @@ static Tk_OptionSpec checkbuttonOptionSpecs[] = {
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
 	DEF_BUTTON_BORDER_WIDTH, Tk_Offset(TkButton, borderWidthPtr),
 	Tk_Offset(TkButton, borderWidth), 0, 0, 0},
-    {TK_OPTION_STRING, "-command", "command", "Command",
-	DEF_BUTTON_COMMAND, Tk_Offset(TkButton, commandPtr), -1,
+    {TK_OPTION_CALLBACK, "-command", "command", "Command",
+	DEF_BUTTON_COMMAND, -1, Tk_Offset(TkButton, commandPtr),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING_TABLE, "-compound", "compound", "Compound",
 	 DEF_BUTTON_COMPOUND, -1, Tk_Offset(TkButton, compound), 0,
@@ -343,13 +343,13 @@ static Tk_OptionSpec checkbuttonOptionSpecs[] = {
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING, "-text", "text", "Text",
 	DEF_BUTTON_TEXT, Tk_Offset(TkButton, textPtr), -1, 0, 0, 0},
-    {TK_OPTION_STRING, "-textvariable", "textVariable", "Variable",
-	DEF_BUTTON_TEXT_VARIABLE, Tk_Offset(TkButton, textVarNamePtr), -1,
+    {TK_OPTION_SCALARVAR, "-textvariable", "textVariable", "Variable",
+	DEF_BUTTON_TEXT_VARIABLE, -1, Tk_Offset(TkButton, textVarNamePtr),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_INT, "-underline", "underline", "Underline",
 	DEF_BUTTON_UNDERLINE, -1, Tk_Offset(TkButton, underline), 0, 0, 0},
-    {TK_OPTION_STRING, "-variable", "variable", "Variable",
-	DEF_CHECKBUTTON_VARIABLE, Tk_Offset(TkButton, selVarNamePtr), -1,
+    {TK_OPTION_SCALARVAR, "-variable", "variable", "Variable",
+	DEF_CHECKBUTTON_VARIABLE, -1, Tk_Offset(TkButton, selVarNamePtr),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING, "-width", "width", "Width",
 	DEF_BUTTON_WIDTH, Tk_Offset(TkButton, widthPtr), -1, 0, 0, 0},
@@ -382,8 +382,8 @@ static Tk_OptionSpec radiobuttonOptionSpecs[] = {
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
 	DEF_BUTTON_BORDER_WIDTH, Tk_Offset(TkButton, borderWidthPtr),
 	Tk_Offset(TkButton, borderWidth), 0, 0, 0},
-    {TK_OPTION_STRING, "-command", "command", "Command",
-	DEF_BUTTON_COMMAND, Tk_Offset(TkButton, commandPtr), -1,
+    {TK_OPTION_CALLBACK, "-command", "command", "Command",
+	DEF_BUTTON_COMMAND, -1, Tk_Offset(TkButton, commandPtr),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING_TABLE, "-compound", "compound", "Compound",
 	 DEF_BUTTON_COMPOUND, -1, Tk_Offset(TkButton, compound), 0,
@@ -449,15 +449,15 @@ static Tk_OptionSpec radiobuttonOptionSpecs[] = {
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING, "-text", "text", "Text",
 	DEF_BUTTON_TEXT, Tk_Offset(TkButton, textPtr), -1, 0, 0, 0},
-    {TK_OPTION_STRING, "-textvariable", "textVariable", "Variable",
-	DEF_BUTTON_TEXT_VARIABLE, Tk_Offset(TkButton, textVarNamePtr), -1,
+    {TK_OPTION_SCALARVAR, "-textvariable", "textVariable", "Variable",
+	DEF_BUTTON_TEXT_VARIABLE, -1, Tk_Offset(TkButton, textVarNamePtr),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_INT, "-underline", "underline", "Underline",
 	DEF_BUTTON_UNDERLINE, -1, Tk_Offset(TkButton, underline), 0, 0, 0},
     {TK_OPTION_STRING, "-value", "value", "Value",
 	DEF_BUTTON_VALUE, Tk_Offset(TkButton, onValuePtr), -1, 0, 0, 0},
-    {TK_OPTION_STRING, "-variable", "variable", "Variable",
-	DEF_RADIOBUTTON_VARIABLE, Tk_Offset(TkButton, selVarNamePtr), -1,
+    {TK_OPTION_SCALARVAR, "-variable", "variable", "Variable",
+	DEF_RADIOBUTTON_VARIABLE, -1, Tk_Offset(TkButton, selVarNamePtr),
 	0, 0, 0},
     {TK_OPTION_STRING, "-width", "width", "Width",
 	DEF_BUTTON_WIDTH, Tk_Offset(TkButton, widthPtr), -1, 0, 0, 0},
@@ -526,10 +526,10 @@ static void		ButtonSelectImageProc _ANSI_ARGS_((
 			    ClientData clientData, int x, int y, int width,
 			    int height, int imgWidth, int imgHeight));
 static char *		ButtonTextVarProc _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, Var name1, 
+			    Tcl_Interp *interp, Var name1,
 			    CONST char *name2, int flags));
 static char *		ButtonVarProc _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, Var name1, 
+			    Tcl_Interp *interp, Var name1,
 			    CONST char *name2, int flags));
 static int		ButtonWidgetObjCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int objc,
@@ -701,6 +701,7 @@ ButtonCreate(clientData, interp, objc, objv, type)
     butPtr->normalTextGC = None;
     butPtr->activeTextGC = None;
     butPtr->disabledGC = None;
+    butPtr->stippleGC = None;
     butPtr->gray = None;
     butPtr->copyGC = None;
     butPtr->widthPtr = NULL;
@@ -982,6 +983,9 @@ DestroyButton(butPtr)
     if (butPtr->disabledGC != None) {
 	Tk_FreeGC(butPtr->display, butPtr->disabledGC);
     }
+    if (butPtr->stippleGC != None) {
+	Tk_FreeGC(butPtr->display, butPtr->stippleGC);
+    }
     if (butPtr->gray != None) {
 	Tk_FreeBitmap(butPtr->display, butPtr->gray);
     }
@@ -1078,6 +1082,13 @@ ConfigureButton(interp, butPtr, objc, objv)
 	    Tk_RestoreSavedOptions(&savedOptions);
 	}
 
+	if ((butPtr->flags & BUTTON_DELETED)) {
+	    /*
+	     * Somehow button was deleted - just abort now. [Bug #824479]
+	     */
+	    return TCL_ERROR;
+	}
+
 	/*
 	 * A few options need special processing, such as setting the
 	 * background from a 3-D border, or filling in complicated
@@ -1107,9 +1118,7 @@ ConfigureButton(interp, butPtr, objc, objv)
 	    Tcl_Obj *valuePtr, *namePtr;
 
 	    if (butPtr->selVarNamePtr == NULL) {
-		butPtr->selVarNamePtr = Tcl_NewStringObj(
-			Tk_Name(butPtr->tkwin), -1);
-		Tcl_IncrRefCount(butPtr->selVarNamePtr);
+		butPtr->selVarNamePtr = LangFindVar(interp, butPtr->tkwin, Tk_Name(butPtr->tkwin));
 	    }
 	    namePtr = butPtr->selVarNamePtr;
 
@@ -1339,20 +1348,15 @@ TkButtonWorldChanged(instanceData)
 	butPtr->activeTextGC = newGC;
     }
 
-    /*
-     * Allocate the disabled graphics context, for drawing the widget in
-     * its disabled state
-     */
     gcValues.background = Tk_3DBorderColor(butPtr->normalBorder)->pixel;
-    if ((butPtr->disabledFg != NULL) && (butPtr->imagePtr == NULL)) {
-	gcValues.foreground = butPtr->disabledFg->pixel;
-	mask = GCForeground | GCBackground | GCFont;
-    } else {
+
+    /*
+     * Create the GC that can be used for stippling
+     */
+
+    if (butPtr->stippleGC == None) {
 	gcValues.foreground = gcValues.background;
 	mask = GCForeground;
-	if (butPtr->compound != COMPOUND_NONE) {
-	    mask |= GCFont;
-	}
 	if (butPtr->gray == None) {
 	    butPtr->gray = Tk_GetBitmap(NULL, butPtr->tkwin, "gray50");
 	}
@@ -1361,6 +1365,19 @@ TkButtonWorldChanged(instanceData)
 	    gcValues.stipple = butPtr->gray;
 	    mask |= GCFillStyle | GCStipple;
 	}
+	butPtr->stippleGC = Tk_GetGC(butPtr->tkwin, mask, &gcValues);
+    }
+
+    /*
+     * Allocate the disabled graphics context, for drawing text in
+     * its disabled state.
+     */
+
+    mask = GCForeground | GCBackground | GCFont;
+    if (butPtr->disabledFg != NULL) {
+	gcValues.foreground = butPtr->disabledFg->pixel;
+    } else {
+	gcValues.foreground = gcValues.background;
     }
     newGC = Tk_GetGC(butPtr->tkwin, mask, &gcValues);
     if (butPtr->disabledGC != None) {
