@@ -1528,13 +1528,13 @@ Tk_WinfoObjCmd(clientData, interp, objc, objv)
 		Tcl_WrongNumArgs(interp, 2, objv, "?-displayof window? id");
 		return TCL_ERROR;
 	    }
-	    string = Tcl_GetStringFromObj(objv[2 + skip], NULL);
-	    if (TkpScanWindowId(interp, Tcl_GetString(objv[2+skip]), &id) != TCL_OK) {
+	    if (TkpScanWindowId(interp, objv[2+skip], &id) != TCL_OK) {
 		return TCL_ERROR;
 	    }
 	    winPtr = (TkWindow *)Tk_IdToWindow(Tk_Display(tkwin), id);
 	    if ((winPtr == NULL) ||
 		    (winPtr->mainPtr != ((TkWindow *) tkwin)->mainPtr)) {
+	        string = Tcl_GetStringFromObj(objv[2 + skip], NULL);
 		Tcl_AppendStringsToObj(resultPtr, "window id \"", string,
 			"\" doesn't exist in this application", (char *) NULL);
 		return TCL_ERROR;
