@@ -1,15 +1,15 @@
 /*
  * tkObj.c --
  *
- *	This file contains procedures that implement the common Tk object
- *	types
+ *      This file contains procedures that implement the common Tk object
+ *      types
  *
  * Copyright (c) 1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkObj.c,v 1.7 2001/08/21 14:43:08 dkf Exp $
+ * RCS: @(#) $Id: tkObj.c,v 1.8 2003/01/28 20:39:16 jenglish Exp $
  */
 
 #include "tkInt.h"
@@ -25,21 +25,21 @@ typedef struct PixelRep {
     int returnValue;
 } PixelRep;
 
-#define SIMPLE_PIXELREP(objPtr)				\
+#define SIMPLE_PIXELREP(objPtr)                         \
     (TclObjInternal(objPtr)->twoPtrValue.ptr2 == 0)
 
-#define SET_SIMPLEPIXEL(objPtr, intval)			\
-    TclObjInternal(objPtr)->twoPtrValue.ptr1 = (VOID *) (intval);	\
+#define SET_SIMPLEPIXEL(objPtr, intval)                 \
+    TclObjInternal(objPtr)->twoPtrValue.ptr1 = (VOID *) (intval);       \
     TclObjInternal(objPtr)->twoPtrValue.ptr2 = 0
 
-#define GET_SIMPLEPIXEL(objPtr)				\
+#define GET_SIMPLEPIXEL(objPtr)                         \
     ((int) TclObjInternal(objPtr)->twoPtrValue.ptr1)
 
-#define SET_COMPLEXPIXEL(objPtr, repPtr)		\
-    TclObjInternal(objPtr)->twoPtrValue.ptr1 = 0;		\
+#define SET_COMPLEXPIXEL(objPtr, repPtr)                \
+    TclObjInternal(objPtr)->twoPtrValue.ptr1 = 0;               \
     TclObjInternal(objPtr)->twoPtrValue.ptr2 = (VOID *) repPtr
 
-#define GET_COMPLEXPIXEL(objPtr)			\
+#define GET_COMPLEXPIXEL(objPtr)                        \
     ((PixelRep *) TclObjInternal(objPtr)->twoPtrValue.ptr2)
 
 
@@ -56,33 +56,35 @@ typedef struct MMRep {
 
 /*
  * The following structure is the internal representation for window objects.
+ * A WindowRep caches name-to-window lookups.  The cache is invalid
+ * if tkwin is NULL or if mainPtr->deletionEpoch does not match epoch.
  */
-
 typedef struct WindowRep {
-    Tk_Window tkwin;
-    Tk_Window mainwin;
-    long epoch;
+    Tk_Window tkwin;            /* Cached window; NULL if not found */
+    TkMainInfo *mainPtr;        /* MainWindow associated with tkwin */
+    long epoch;                 /* Value of mainPtr->deletionEpoch at last
+				 * successful lookup.  */
 } WindowRep;
 
 /*
  * Prototypes for procedures defined later in this file:
  */
 
-static void		DupMMInternalRep _ANSI_ARGS_((Tcl_Obj *srcPtr,
+static void             DupMMInternalRep _ANSI_ARGS_((Tcl_Obj *srcPtr,
 			    Tcl_Obj *copyPtr));
-static void		DupPixelInternalRep _ANSI_ARGS_((Tcl_Obj *srcPtr,
+static void             DupPixelInternalRep _ANSI_ARGS_((Tcl_Obj *srcPtr,
 			    Tcl_Obj *copyPtr));
-static void		DupWindowInternalRep _ANSI_ARGS_((Tcl_Obj *srcPtr,
+static void             DupWindowInternalRep _ANSI_ARGS_((Tcl_Obj *srcPtr,
 			    Tcl_Obj *copyPtr));
-static void		FreeMMInternalRep _ANSI_ARGS_((Tcl_Obj *objPtr));
-static void		FreePixelInternalRep _ANSI_ARGS_((Tcl_Obj *objPtr));
-static void		FreeWindowInternalRep _ANSI_ARGS_((Tcl_Obj *objPtr));
-static void		UpdateStringOfMM _ANSI_ARGS_((Tcl_Obj *objPtr));
-static int		SetMMFromAny _ANSI_ARGS_((Tcl_Interp *interp,
+static void             FreeMMInternalRep _ANSI_ARGS_((Tcl_Obj *objPtr));
+static void             FreePixelInternalRep _ANSI_ARGS_((Tcl_Obj *objPtr));
+static void             FreeWindowInternalRep _ANSI_ARGS_((Tcl_Obj *objPtr));
+static void             UpdateStringOfMM _ANSI_ARGS_((Tcl_Obj *objPtr));
+static int              SetMMFromAny _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *objPtr));
-static int		SetPixelFromAny _ANSI_ARGS_((Tcl_Interp *interp,
+static int              SetPixelFromAny _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *objPtr));
-static int		SetWindowFromAny _ANSI_ARGS_((Tcl_Interp *interp,
+static int              SetWindowFromAny _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Obj *objPtr));
 
 /*
@@ -92,11 +94,11 @@ static int		SetWindowFromAny _ANSI_ARGS_((Tcl_Interp *interp,
  */
 
 static Tcl_ObjType pixelObjType = {
-    "pixel",			/* name */
-    FreePixelInternalRep,	/* freeIntRepProc */
-    DupPixelInternalRep,	/* dupIntRepProc */
-    NULL,			/* updateStringProc */
-    SetPixelFromAny		/* setFromAnyProc */
+    "pixel",                    /* name */
+    FreePixelInternalRep,       /* freeIntRepProc */
+    DupPixelInternalRep,        /* dupIntRepProc */
+    NULL,                       /* updateStringProc */
+    SetPixelFromAny             /* setFromAnyProc */
 };
 
 /*
@@ -106,11 +108,11 @@ static Tcl_ObjType pixelObjType = {
  */
 
 static Tcl_ObjType mmObjType = {
-    "mm",			/* name */
-    FreeMMInternalRep,		/* freeIntRepProc */
-    DupMMInternalRep,		/* dupIntRepProc */
-    UpdateStringOfMM,		/* updateStringProc */
-    SetMMFromAny		/* setFromAnyProc */
+    "mm",                       /* name */
+    FreeMMInternalRep,          /* freeIntRepProc */
+    DupMMInternalRep,           /* dupIntRepProc */
+    UpdateStringOfMM,           /* updateStringProc */
+    SetMMFromAny                /* setFromAnyProc */
 };
 
 /*
@@ -119,11 +121,11 @@ static Tcl_ObjType mmObjType = {
  */
 
 static Tcl_ObjType windowObjType = {
-    "window",				/* name */
-    FreeWindowInternalRep,		/* freeIntRepProc */
-    DupWindowInternalRep,		/* dupIntRepProc */
-    NULL,				/* updateStringProc */
-    SetWindowFromAny			/* setFromAnyProc */
+    "window",                           /* name */
+    FreeWindowInternalRep,              /* freeIntRepProc */
+    DupWindowInternalRep,               /* dupIntRepProc */
+    NULL,                               /* updateStringProc */
+    SetWindowFromAny                    /* setFromAnyProc */
 };
 
 
@@ -133,34 +135,34 @@ static Tcl_ObjType windowObjType = {
  *
  * Tk_GetPixelsFromObj --
  *
- *	Attempt to return a pixel value from the Tcl object "objPtr". If the
- *	object is not already a pixel value, an attempt will be made to convert
- *	it to one.
+ *      Attempt to return a pixel value from the Tcl object "objPtr". If the
+ *      object is not already a pixel value, an attempt will be made to convert
+ *      it to one.
  *
  * Results:
- *	The return value is a standard Tcl object result. If an error occurs
- *	during conversion, an error message is left in the interpreter's
- *	result unless "interp" is NULL.
+ *      The return value is a standard Tcl object result. If an error occurs
+ *      during conversion, an error message is left in the interpreter's
+ *      result unless "interp" is NULL.
  *
  * Side effects:
- *	If the object is not already a pixel, the conversion will free
- *	any old internal representation.
+ *      If the object is not already a pixel, the conversion will free
+ *      any old internal representation.
  *
  *----------------------------------------------------------------------
  */
 
 int
 Tk_GetPixelsFromObj(interp, tkwin, objPtr, intPtr)
-    Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
+    Tcl_Interp *interp;         /* Used for error reporting if not NULL. */
     Tk_Window tkwin;
-    Tcl_Obj *objPtr;		/* The object from which to get pixels. */
-    int *intPtr;		/* Place to store resulting pixels. */
+    Tcl_Obj *objPtr;            /* The object from which to get pixels. */
+    int *intPtr;                /* Place to store resulting pixels. */
 {
     int result;
     double d;
     PixelRep *pixelPtr;
     static double bias[] = {
-	1.0,	10.0,	25.4,	25.4 / 72.0
+	1.0,    10.0,   25.4,   25.4 / 72.0
     };
 
     if (objPtr->typePtr != &pixelObjType) {
@@ -187,7 +189,7 @@ Tk_GetPixelsFromObj(interp, tkwin, objPtr, intPtr)
 	    }
 	    pixelPtr->tkwin = tkwin;
 	}
-        *intPtr = pixelPtr->returnValue;
+	*intPtr = pixelPtr->returnValue;
     }
     return TCL_OK;
 }
@@ -197,22 +199,22 @@ Tk_GetPixelsFromObj(interp, tkwin, objPtr, intPtr)
  *
  * FreePixelInternalRep --
  *
- *	Deallocate the storage associated with a pixel object's internal
- *	representation.
+ *      Deallocate the storage associated with a pixel object's internal
+ *      representation.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Frees objPtr's internal representation and sets objPtr's
- *	internalRep to NULL.
+ *      Frees objPtr's internal representation and sets objPtr's
+ *      internalRep to NULL.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 FreePixelInternalRep(objPtr)
-    Tcl_Obj *objPtr;		/* Pixel object with internal rep to free. */
+    Tcl_Obj *objPtr;            /* Pixel object with internal rep to free. */
 {
     PixelRep *pixelPtr;
 
@@ -221,6 +223,7 @@ FreePixelInternalRep(objPtr)
 	ckfree((char *) pixelPtr);
     }
     SET_SIMPLEPIXEL(objPtr, 0);
+    objPtr->typePtr = NULL;
 }
 
 /*
@@ -228,23 +231,23 @@ FreePixelInternalRep(objPtr)
  *
  * DupPixelInternalRep --
  *
- *	Initialize the internal representation of a pixel Tcl_Obj to a
- *	copy of the internal representation of an existing pixel object.
+ *      Initialize the internal representation of a pixel Tcl_Obj to a
+ *      copy of the internal representation of an existing pixel object.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	copyPtr's internal rep is set to the pixel corresponding to
- *	srcPtr's internal rep.
+ *      copyPtr's internal rep is set to the pixel corresponding to
+ *      srcPtr's internal rep.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 DupPixelInternalRep(srcObjPtr, copyObjPtr)
-    register Tcl_Obj *srcObjPtr;	/* Object with internal rep to copy. */
-    register Tcl_Obj *copyObjPtr;	/* Object with internal rep to set. */
+    register Tcl_Obj *srcObjPtr;        /* Object with internal rep to copy. */
+    register Tcl_Obj *copyObjPtr;       /* Object with internal rep to set. */
 {
     PixelRep *oldPtr, *newPtr;
 
@@ -268,25 +271,25 @@ DupPixelInternalRep(srcObjPtr, copyObjPtr)
  *
  * SetPixelFromAny --
  *
- *	Attempt to generate a pixel internal form for the Tcl object
- *	"objPtr".
+ *      Attempt to generate a pixel internal form for the Tcl object
+ *      "objPtr".
  *
  * Results:
- *	The return value is a standard Tcl result. If an error occurs during
- *	conversion, an error message is left in the interpreter's result
- *	unless "interp" is NULL.
+ *      The return value is a standard Tcl result. If an error occurs during
+ *      conversion, an error message is left in the interpreter's result
+ *      unless "interp" is NULL.
  *
  * Side effects:
- *	If no error occurs, a pixel representation of the object is
- *	stored internally and the type of "objPtr" is set to pixel.
+ *      If no error occurs, a pixel representation of the object is
+ *      stored internally and the type of "objPtr" is set to pixel.
  *
  *----------------------------------------------------------------------
  */
 
 static int
 SetPixelFromAny(interp, objPtr)
-    Tcl_Interp *interp;		/* Used for error reporting if not NULL. */
-    Tcl_Obj *objPtr;		/* The object to convert. */
+    Tcl_Interp *interp;         /* Used for error reporting if not NULL. */
+    Tcl_Obj *objPtr;            /* The object to convert. */
 {
     Tcl_ObjType *typePtr;
     char *string, *rest;
@@ -369,34 +372,34 @@ SetPixelFromAny(interp, objPtr)
  *
  * Tk_GetMMFromObj --
  *
- *	Attempt to return an mm value from the Tcl object "objPtr". If the
- *	object is not already an mm value, an attempt will be made to convert
- *	it to one.
+ *      Attempt to return an mm value from the Tcl object "objPtr". If the
+ *      object is not already an mm value, an attempt will be made to convert
+ *      it to one.
  *
  * Results:
- *	The return value is a standard Tcl object result. If an error occurs
- *	during conversion, an error message is left in the interpreter's
- *	result unless "interp" is NULL.
+ *      The return value is a standard Tcl object result. If an error occurs
+ *      during conversion, an error message is left in the interpreter's
+ *      result unless "interp" is NULL.
  *
  * Side effects:
- *	If the object is not already a pixel, the conversion will free
- *	any old internal representation.
+ *      If the object is not already a pixel, the conversion will free
+ *      any old internal representation.
  *
  *----------------------------------------------------------------------
  */
 
 int
 Tk_GetMMFromObj(interp, tkwin, objPtr, doublePtr)
-    Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
+    Tcl_Interp *interp;         /* Used for error reporting if not NULL. */
     Tk_Window tkwin;
-    Tcl_Obj *objPtr;		/* The object from which to get mms. */
-    double *doublePtr;		/* Place to store resulting millimeters. */
+    Tcl_Obj *objPtr;            /* The object from which to get mms. */
+    double *doublePtr;          /* Place to store resulting millimeters. */
 {
     int result;
     double d;
     MMRep *mmPtr;
     static double bias[] = {
-	10.0,	25.4,	1.0,	25.4 / 72.0
+	10.0,   25.4,   1.0,    25.4 / 72.0
     };
 
     if (objPtr->typePtr != &mmObjType) {
@@ -428,25 +431,26 @@ Tk_GetMMFromObj(interp, tkwin, objPtr, doublePtr)
  *
  * FreeMMInternalRep --
  *
- *	Deallocate the storage associated with a mm object's internal
- *	representation.
+ *      Deallocate the storage associated with a mm object's internal
+ *      representation.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Frees objPtr's internal representation and sets objPtr's
- *	internalRep to NULL.
+ *      Frees objPtr's internal representation and sets objPtr's
+ *      internalRep to NULL.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 FreeMMInternalRep(objPtr)
-    Tcl_Obj *objPtr;		/* MM object with internal rep to free. */
+    Tcl_Obj *objPtr;            /* MM object with internal rep to free. */
 {
     ckfree((char *) objPtr->internalRep.otherValuePtr);
     objPtr->internalRep.otherValuePtr = NULL;
+    objPtr->typePtr = NULL;
 }
 
 /*
@@ -454,23 +458,23 @@ FreeMMInternalRep(objPtr)
  *
  * DupMMInternalRep --
  *
- *	Initialize the internal representation of a pixel Tcl_Obj to a
- *	copy of the internal representation of an existing pixel object.
+ *      Initialize the internal representation of a pixel Tcl_Obj to a
+ *      copy of the internal representation of an existing pixel object.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	copyPtr's internal rep is set to the pixel corresponding to
- *	srcPtr's internal rep.
+ *      copyPtr's internal rep is set to the pixel corresponding to
+ *      srcPtr's internal rep.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 DupMMInternalRep(srcObjPtr, copyObjPtr)
-    register Tcl_Obj *srcObjPtr;	/* Object with internal rep to copy. */
-    register Tcl_Obj *copyObjPtr;	/* Object with internal rep to set. */
+    register Tcl_Obj *srcObjPtr;        /* Object with internal rep to copy. */
+    register Tcl_Obj *copyObjPtr;       /* Object with internal rep to set. */
 {
     MMRep *oldPtr, *newPtr;
 
@@ -516,7 +520,7 @@ UpdateStringOfMM(objPtr)
     mmPtr = (MMRep *) objPtr->internalRep.otherValuePtr;
     /* assert( mmPtr->units == -1 && objPtr->bytes == NULL ); */
     if ((mmPtr->units != -1) || (objPtr->bytes != NULL)) {
-        panic("UpdateStringOfMM: false precondition");
+	panic("UpdateStringOfMM: false precondition");
     }
 
     Tcl_PrintDouble((Tcl_Interp *) NULL, mmPtr->value, buffer);
@@ -536,25 +540,25 @@ UpdateStringOfMM(objPtr)
  *
  * SetMMFromAny --
  *
- *	Attempt to generate a mm internal form for the Tcl object
- *	"objPtr".
+ *      Attempt to generate a mm internal form for the Tcl object
+ *      "objPtr".
  *
  * Results:
- *	The return value is a standard Tcl result. If an error occurs during
- *	conversion, an error message is left in the interpreter's result
- *	unless "interp" is NULL.
+ *      The return value is a standard Tcl result. If an error occurs during
+ *      conversion, an error message is left in the interpreter's result
+ *      unless "interp" is NULL.
  *
  * Side effects:
- *	If no error occurs, a mm representation of the object is
- *	stored internally and the type of "objPtr" is set to mm.
+ *      If no error occurs, a mm representation of the object is
+ *      stored internally and the type of "objPtr" is set to mm.
  *
  *----------------------------------------------------------------------
  */
 
 static int
 SetMMFromAny(interp, objPtr)
-    Tcl_Interp *interp;		/* Used for error reporting if not NULL. */
-    Tcl_Obj *objPtr;		/* The object to convert. */
+    Tcl_Interp *interp;         /* Used for error reporting if not NULL. */
+    Tcl_Obj *objPtr;            /* The object to convert. */
 {
     Tcl_ObjType *typePtr;
     char *string, *rest;
@@ -604,14 +608,14 @@ SetMMFromAny(interp, objPtr)
 	     */
 
 	    error:
-            Tcl_AppendResult(interp, "bad screen distance \"", string,
-                    "\"", (char *) NULL);
-            return TCL_ERROR;
-        }
-        while ((*rest != '\0') && isspace(UCHAR(*rest))) {
-            rest++;
-        }
-        switch (*rest) {
+	    Tcl_AppendResult(interp, "bad screen distance \"", string,
+		    "\"", (char *) NULL);
+	    return TCL_ERROR;
+	}
+	while ((*rest != '\0') && isspace(UCHAR(*rest))) {
+	    rest++;
+	}
+	switch (*rest) {
 	    case '\0':
 		units = -1;
 		break;
@@ -646,13 +650,13 @@ SetMMFromAny(interp, objPtr)
 	(*typePtr->freeIntRepProc)(objPtr);
     }
 
-    objPtr->typePtr	= &mmObjType;
+    objPtr->typePtr     = &mmObjType;
 
-    mmPtr		= (MMRep *) ckalloc(sizeof(MMRep));
-    mmPtr->value	= d;
-    mmPtr->units	= units;
-    mmPtr->tkwin	= NULL;
-    mmPtr->returnValue	= d;
+    mmPtr               = (MMRep *) ckalloc(sizeof(MMRep));
+    mmPtr->value        = d;
+    mmPtr->units        = units;
+    mmPtr->tkwin        = NULL;
+    mmPtr->returnValue  = d;
 
     objPtr->internalRep.otherValuePtr = (VOID *) mmPtr;
 
@@ -664,61 +668,58 @@ SetMMFromAny(interp, objPtr)
  *
  * TkGetWindowFromObj --
  *
- *	Attempt to return a Tk_Window from the Tcl object "objPtr". If the
- *	object is not already a Tk_Window, an attempt will be made to convert
- *	it to one.
+ *      Attempt to return a Tk_Window from the Tcl object "objPtr". If the
+ *      object is not already a Tk_Window, an attempt will be made to convert
+ *      it to one.
  *
  * Results:
- *	The return value is a standard Tcl object result. If an error occurs
- *	during conversion, an error message is left in the interpreter's
- *	result unless "interp" is NULL.
+ *      The return value is a standard Tcl object result. If an error occurs
+ *      during conversion, an error message is left in the interpreter's
+ *      result unless "interp" is NULL.
  *
  * Side effects:
- *	If the object is not already a Tk_Window, the conversion will free
- *	any old internal representation.
+ *      If the object is not already a Tk_Window, the conversion will free
+ *      any old internal representation.
  *
  *----------------------------------------------------------------------
  */
 
 int
 TkGetWindowFromObj(interp, tkwin, objPtr, windowPtr)
-    Tcl_Interp *interp; 	/* Used for error reporting if not NULL. */
-    Tk_Window tkwin;		/* A token to get the main window from. */
-    Tcl_Obj *objPtr;		/* The object from which to get boolean. */
-    Tk_Window *windowPtr;	/* Place to store resulting window. */
+    Tcl_Interp *interp;         /* Used for error reporting if not NULL. */
+    Tk_Window tkwin;            /* A token to get the main window from. */
+    Tcl_Obj *objPtr;            /* The object from which to get window. */
+    Tk_Window *windowPtr;       /* Place to store resulting window. */
 {
+    TkMainInfo *mainPtr = ((TkWindow *)tkwin)->mainPtr;
     register WindowRep *winPtr;
-    TkDisplay *dispPtr = ((TkWindow *)tkwin)->dispPtr;
-    Tk_Window foundWindow;
+    int result;
 
-    if (objPtr->typePtr != &windowObjType) {
-	register int result = SetWindowFromAny(interp, objPtr);
-	if (result != TCL_OK) {
-	    return result;
-	}
+    result = Tcl_ConvertToType(interp, objPtr, &windowObjType);
+    if (result != TCL_OK) {
+	return result;
     }
 
     winPtr = (WindowRep *) objPtr->internalRep.otherValuePtr;
-    if (winPtr == NULL) {
-	winPtr = (WindowRep *) ckalloc(sizeof(WindowRep));
-	objPtr->internalRep.otherValuePtr = (VOID *) winPtr;
-	goto parseWindowString;
-
-    } else if (tkwin != winPtr->mainwin ||
-	       dispPtr->deletionEpoch != winPtr->epoch) {
-    parseWindowString:
-	foundWindow = Tk_NameToWindow(interp,
+    if (    winPtr->tkwin == NULL
+	 || winPtr->mainPtr == NULL
+	 || winPtr->mainPtr != mainPtr
+	 || winPtr->epoch != mainPtr->deletionEpoch)
+    {
+	/* Cache is invalid.
+	 */
+	winPtr->tkwin = Tk_NameToWindow(interp,
 		Tcl_GetStringFromObj(objPtr, NULL), tkwin);
-	if (foundWindow == NULL) {
-	    return TCL_ERROR;
-	}
-
-	winPtr->tkwin = foundWindow;
-	winPtr->mainwin = tkwin;
-	winPtr->epoch = dispPtr->deletionEpoch;
+	winPtr->mainPtr = mainPtr;
+	winPtr->epoch = mainPtr ? mainPtr->deletionEpoch : 0;
     }
 
     *windowPtr = winPtr->tkwin;
+
+    if (winPtr->tkwin == NULL) {
+	/* ASSERT: Tk_NameToWindow has left error message in interp */
+	return TCL_ERROR;
+    }
     return TCL_OK;
 }
 
@@ -726,28 +727,28 @@ TkGetWindowFromObj(interp, tkwin, objPtr, windowPtr)
  *----------------------------------------------------------------------
  *
  * SetWindowFromAny --
- *
- *	Attempt to generate a Tk_Window internal form for the Tcl object
- *	"objPtr".
+ *      Generate a windowObj internal form for the Tcl object "objPtr".
  *
  * Results:
- *	The return value is a standard Tcl result. If an error occurs during
- *	conversion, an error message is left in the interpreter's result
- *	unless "interp" is NULL.
+ *      Always returns TCL_OK.
  *
  * Side effects:
- *	If no error occurs, a standard window value is stored as "objPtr"s
- *	internal representation and the type of "objPtr" is set to Tk_Window.
+ *      Sets objPtr's internal representation to an uninitialized
+ *      windowObj. Frees the old internal representation, if any.
+ *
+ * See also:
+ *      TkGetWindowFromObj, which initializes the WindowRep cache.
  *
  *----------------------------------------------------------------------
  */
 
 static int
 SetWindowFromAny(interp, objPtr)
-    Tcl_Interp *interp;		/* Used for error reporting if not NULL. */
-    register Tcl_Obj *objPtr;	/* The object to convert. */
+    Tcl_Interp *interp;         /* Used for error reporting if not NULL. */
+    register Tcl_Obj *objPtr;   /* The object to convert. */
 {
     Tcl_ObjType *typePtr;
+    WindowRep *winPtr;
 
     /*
      * Free the old internalRep before setting the new one.
@@ -758,9 +759,14 @@ SetWindowFromAny(interp, objPtr)
     if ((typePtr != NULL) && (typePtr->freeIntRepProc != NULL)) {
 	(*typePtr->freeIntRepProc)(objPtr);
     }
+
+    winPtr = (WindowRep *) ckalloc(sizeof(WindowRep));
+    winPtr->tkwin = NULL;
+    winPtr->mainPtr = NULL;
+    winPtr->epoch = 0;
+
+    objPtr->internalRep.otherValuePtr = (VOID*)winPtr;
     TclObjSetType(objPtr,&windowObjType);
-    objPtr->internalRep.twoPtrValue.ptr1 = NULL;
-    objPtr->internalRep.twoPtrValue.ptr2 = NULL;
 
     return TCL_OK;
 }
@@ -770,15 +776,15 @@ SetWindowFromAny(interp, objPtr)
  *
  * DupWindowInternalRep --
  *
- *	Initialize the internal representation of a window Tcl_Obj to a
- *	copy of the internal representation of an existing window object.
+ *      Initialize the internal representation of a window Tcl_Obj to a
+ *      copy of the internal representation of an existing window object.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	copyPtr's internal rep is set to refer to the same window as
- *	srcPtr's internal rep.
+ *      copyPtr's internal rep is set to refer to the same window as
+ *      srcPtr's internal rep.
  *
  *----------------------------------------------------------------------
  */
@@ -790,17 +796,13 @@ DupWindowInternalRep(srcObjPtr, copyObjPtr)
 {
     register WindowRep *oldPtr, *newPtr;
 
-    copyObjPtr->typePtr = srcObjPtr->typePtr;
     oldPtr = srcObjPtr->internalRep.otherValuePtr;
-    if (oldPtr == NULL) {
-	copyObjPtr->internalRep.otherValuePtr = NULL;
-    } else {
-	newPtr = (WindowRep *) ckalloc(sizeof(WindowRep));
-	newPtr->tkwin = oldPtr->tkwin;
-	newPtr->mainwin = oldPtr->mainwin;
-	newPtr->epoch = oldPtr->epoch;
-	copyObjPtr->internalRep.otherValuePtr = (VOID *)newPtr;
-    }
+    newPtr = (WindowRep *) ckalloc(sizeof(WindowRep));
+    newPtr->tkwin = oldPtr->tkwin;
+    newPtr->mainPtr = oldPtr->mainPtr;
+    newPtr->epoch = oldPtr->epoch;
+    copyObjPtr->internalRep.otherValuePtr = (VOID *)newPtr;
+    copyObjPtr->typePtr = srcObjPtr->typePtr;
 }
 
 /*
@@ -808,27 +810,26 @@ DupWindowInternalRep(srcObjPtr, copyObjPtr)
  *
  * FreeWindowInternalRep --
  *
- *	Deallocate the storage associated with a window object's internal
- *	representation.
+ *      Deallocate the storage associated with a window object's internal
+ *      representation.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Frees objPtr's internal representation and sets objPtr's
- *	internalRep to NULL.
+ *      Frees objPtr's internal representation and sets objPtr's
+ *      internalRep to NULL.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 FreeWindowInternalRep(objPtr)
-    Tcl_Obj *objPtr;		/* Window object with internal rep to free. */
+    Tcl_Obj *objPtr;            /* Window object with internal rep to free. */
 {
-    if (objPtr->internalRep.otherValuePtr != NULL) {
-	ckfree((char *) objPtr->internalRep.otherValuePtr);
-	objPtr->internalRep.otherValuePtr = NULL;
-    }
+    ckfree((char *) objPtr->internalRep.otherValuePtr);
+    objPtr->internalRep.otherValuePtr = NULL;
+    objPtr->typePtr = NULL;
 }
 
 /*
@@ -836,14 +837,14 @@ FreeWindowInternalRep(objPtr)
  *
  * TkRegisterObjTypes --
  *
- *	Registers Tk's Tcl_ObjType structures with the Tcl run-time.
+ *      Registers Tk's Tcl_ObjType structures with the Tcl run-time.
  *
  * Results:
- *	None
+ *      None
  *
  * Side effects:
- *	All instances of Tcl_ObjType structues used in Tk are registered
- *	with Tcl.
+ *      All instances of Tcl_ObjType structues used in Tk are registered
+ *      with Tcl.
  *
  *----------------------------------------------------------------------
  */
