@@ -478,7 +478,7 @@ AlignParseProc(clientData, interp, tkwin, arg, widgRec, offset)
  */
 
 	/* ARGSUSED */
-static Arg
+static Tcl_Obj *
 AlignPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;		/* Ignored. */
     Tk_Window tkwin;			/* Window for text widget. */
@@ -489,7 +489,7 @@ AlignPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 					 * information about how to reclaim
 					 * storage for return string. */
 {
-    Arg result = NULL;
+    Tcl_Obj * result = NULL;
     LangSetDefault(&result, "");
     /* freeProc set correct for Tcl */
     *freeProcPtr = TCL_STATIC;
