@@ -53,7 +53,7 @@ EXTERN TIX_DECLARE_SUBCMD(Tix_GrSort);
 
 static int		SortCompareProc _ANSI_ARGS_((CONST VOID *first,
 			    CONST VOID *second));
-Arg			Tix_GrGetCellText _ANSI_ARGS_((WidgetPtr wPtr,
+Tcl_Obj *			Tix_GrGetCellText _ANSI_ARGS_((WidgetPtr wPtr,
 			    int x, int y));
 Tix_GrSortItem *	Tix_GrGetSortItems _ANSI_ARGS_((WidgetPtr wPtr,
 			    int axis, int start, int end, int sortKeyIndex));
@@ -80,7 +80,7 @@ void			Tix_GrFreeSortItems _ANSI_ARGS_((WidgetPtr wPtr,
 
 	/* ARGSUSED */
 
-Arg
+Tcl_Obj *
 Tix_GrGetCellText(wPtr, x, y)
     WidgetPtr wPtr;
     int x;
@@ -367,8 +367,8 @@ SortCompareProc(first, second)
     CONST VOID *first, *second;		/* Elements to be compared. */
 {
     int order;
-    Arg firstString  = ((Tix_GrSortItem*)first )->data;
-    Arg secondString = ((Tix_GrSortItem*)second)->data;
+    Tcl_Obj * firstString  = ((Tix_GrSortItem*)first )->data;
+    Tcl_Obj * secondString = ((Tix_GrSortItem*)second)->data;
 
     order = 0;
     if (sortCode != TCL_OK) {
