@@ -1,5 +1,5 @@
-/* $Id: tixAppInit.c,v 1.2.2.1 2001/11/15 05:18:16 idiscovery Exp $ */
-/*
+/* $Id: tixAppInit.c,v 1.1.2.1 2001/11/15 05:18:16 idiscovery Exp $ */
+/* 
  * tixAppInit.c --
  *
  *	Provides a default version of the Tcl_AppInit procedure for
@@ -23,7 +23,7 @@
 
 extern int matherr();
 int *tclDummyMathPtr = (int *) matherr;
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -49,7 +49,7 @@ main(argc, argv)
     Tk_Main(argc, argv, Tcl_AppInit);
     return 0;			/* Needed only to prevent compiler warning. */
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -73,17 +73,17 @@ int
 Tcl_AppInit(interp)
     Tcl_Interp *interp;		/* Interpreter for application. */
 {
-    Tk_Window main;
-
     if (Tcl_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
     if (Tk_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
+    Tcl_StaticPackage(interp, "Tk",  Tk_Init,  (Tcl_PackageInitProc *) NULL);
     if (Tix_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
+    Tcl_StaticPackage(interp, "Tix", Tix_Init, (Tcl_PackageInitProc *) NULL);
 
     /*
      * Call the init procedures for included packages.  Each call should
@@ -112,6 +112,6 @@ Tcl_AppInit(interp)
 	 if {[file isfile [set file ~/.tixwishrc]]} {source $file}
      */
 
-    tcl_RcFileName = "~/.tixwishrc";
+
     return TCL_OK;
 }
