@@ -82,10 +82,26 @@ DllMain(hInstance, reason, reserved)
      */
 
     if (reason == DLL_PROCESS_ATTACH) {
+        /* We pretend to be Tcl as well - perhaps 
+           we should use perl's or Tk::Event's hInstance for this ?
+         */
 	tclInstance = hInstance;
-        TkWinXInit(hInstance);
+        /* and it is important to set Tk instance so 
+           we can find bitmaps and cursors
+         */
+        TkWinSetHINSTANCE(hInstance);
+        /* don't do this here - (do it in Boot?)
+           as it makes calls to Tcl_Xxxx from Tk::Event and 
+           we have not set vtables yet.
+         */
+#ifndef _LANG  
+         TkWinXInit(hInstance);
+#endif
     } else if (reason == DLL_PROCESS_DETACH) {
-//        TkWinXCleanup(hInstance);
+/* this is done by clean call now */
+#if 0
+      TkWinXCleanup(hInstance);
+#endif
     }
     return(TRUE);
 }
